%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Pradeep Nagesh, Arizona State University, USA (2007-2008)  
%         pnagesh@asu.edu
%         Face Recognition using Joint Sparsity Models
%
% Description: Handles the directory paths. Get the path on which the 
% result has to saved or retrieved from according to the sate of the face
% recognition system (like training, testing etc)
%
%  Input/Output 
%  Opts: Input data structure containing various parameter settings.  
%  Refer to FR_Document.doc for details.
%
%  mode: Tells what mode is the system in according to which the relative
%  paths are determined and returned so that data can be saved/retrived
%  from the path. In both cases the names of the files (to be saved or
%  retrived) are determined according to a naming convention.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ret ret1 ret2 ret3] = GetPath_name(mode, Opts)

%%                Set up variables and names

ret = 0;
ret1 = 0;
ret2 = 0;
ret3 = 0;

checkWord = 'JSM_Face_Recognition';
currentPath = pwd;
currentPath_pwd = pwd;
checkPath = currentPath_pwd(end-19:end);

if ~strcmp(checkPath,checkWord)
    error('Please cd to the correct directory before running the program... cd...\JSM_Face_Recognition');
end
    
DataBaseName = Opts.DataBaseName;
root_path      = strcat(currentPath,'\Database_Training_Image_MATLAB_Files\',DataBaseName,'\'); 

if (strcmp(mode,'Testing Data'))
    ret       = strcat(currentPath,'\TestSet_Database\',DataBaseName,'\');
    return;
end

if (strcmp(mode,'Global Result Save'))
    ret       = strcat(currentPath,'\TestSet_Database\',DataBaseName);
    return;
end

if (strcmp(mode,'TV L1 Data'))
    ret       = strcat(currentPath,'\TestSet_Database\',DataBaseName,'\FT_Data');
    return;
end

%   If SampleFraction is a parameter, then convert the sample fraction as
%   an interger number, that is to be used for naming/getting names
if ~Opts.twoDReconstruction
    sample_num     = Opts.SampleFraction; 
    sample_num     = strcat('_',num2str(sample_num));
end

NumImages      = Opts.M(Opts.ClassNum);
NumImages      = strcat('_',num2str(NumImages));
        
switch (mode)
    case {'Training Images', 'Training Data',...
          'Training Images MS','Training Data MS',...
          'Training Images SJSM','Training Data SJSM',...
          'Training Images MS SJSM','Training Data MS SJSM',...
           'JSM Data Save', 'JSM Data'}
        ClassNum       = Opts.ClassNum;
        ClassNum       = strcat('_',num2str(ClassNum));
        
    case {'Image Data'}
        index          = Opts.index;
        ClassNum       = Opts.ClassNum;
        ClassNum       = strcat('_',num2str(ClassNum));
end

switch (mode)

    case {'Phi'}
        ret            = strcat(root_path, 'RandomMatrix\Phi');
        if (~Opts.twoDReconstruction)
            ret            = strcat(ret,sample_num);
        end
        
    case {'Phi Full'}
        ret            = strcat(root_path, 'Phi_Full');
        
    case {'JSM Data Save', 'JSM Data'}
        if (Opts.twoDReconstruction)
            File_name      = 'JSM_2d';
        else
            File_name      = strcat('JSM',sample_num);
        end
        File_name      = strcat(strcat(File_name,NumImages));
        ret            = strcat(root_path,'JSM_data\ImageClass');
        ret            = strcat(ret,ClassNum);
        ret            = strcat(ret, '\');
        ret            = strcat(ret,File_name);
       
    case {'JSM Test Data'}    
        ret            = strcat(currentPath,'\TestSet_Database\',DataBaseName,'\');
        if (Opts.twoDReconstruction)
            File_name      = 'JSM_data_2d';
        else
            File_name      = strcat('JSM_data',sample_num);
        end
        ret            = strcat(ret,File_name);

    case {'JSM Test Data SJSM'}    
        ret            = strcat(currentPath,'\TestSet_Database\',DataBaseName,'\');
        File_name      = 'JSM_data_SJSM_2d';
        ret            = strcat(ret,File_name);
        
    case {'Training Images', 'Training Data'}
        ret            = strcat(root_path,'JSM_data\ImageClass',ClassNum, '\');
        if (strcmp(mode,'Training Data'))
            if (Opts.twoDReconstruction)
                File_name1     = strcat('Image_2d_M',NumImages);
            else
                File_name1     = strcat('Image','_sf',sample_num,'_M',NumImages);
            end
            ret            = strcat(ret,File_name1);
        else
            % look into this
            temp           = ret;   
            if (Opts.twoDReconstruction)
                File_name1     = strcat('Image_Common_2d_M',NumImages);
                File_name2     = strcat('Image_Inn_2d_M', NumImages);
            else
                File_name1     = strcat('Image_Common','_sf',sample_num,'_M',NumImages);
                File_name2     = strcat('Image_Inn','_sf',sample_num, '_M', NumImages);
            end
            ret            = strcat(temp,File_name1,'.jpg');
            ret1           = strcat(temp,File_name2,'.jpg');
        end

    case {'Training Images MS', 'Training Data MS'}
        ret            = strcat(root_path,'JSM_data\ImageClass',ClassNum, '\');
        if (strcmp(mode,'Training Data MS'))
            File_name1     = strcat('Image_MS_2d_M',NumImages);
            ret            = strcat(ret,File_name1);
        else
            % look into this
            temp           = ret;   
                File_name1     = strcat('Image_MS_Common_2d_M',NumImages);
                File_name2     = strcat('Image_MS_Inn_2d_M', NumImages);
            ret            = strcat(temp,File_name1,'.jpg');
            ret1           = strcat(temp,File_name2,'.jpg');
        end

    case {'Training Images SJSM', 'Training Data SJSM'}
        ret            = strcat(root_path,'JSM_data\ImageClass',ClassNum, '\');
        if (strcmp(mode,'Training Data SJSM'))
            File_name1     = strcat('Image_SJSM_2d_M',NumImages);
            ret            = strcat(ret,File_name1);
        else
            % look into this
            temp           = ret;   
            File_name1     = strcat('Image_SJSM_Common_2d_M',NumImages);
            File_name2     = strcat('Image_SJSM_Inn_2d_M', NumImages);
            ret            = strcat(temp,File_name1,'.jpg');
            ret1           = strcat(temp,File_name2,'.jpg');
        end

    case {'Training Images MS SJSM', 'Training Data MS SJSM'}
        ret            = strcat(root_path,'JSM_data\ImageClass',ClassNum, '\');
        if (strcmp(mode,'Training Data MS SJSM'))
            File_name1     = strcat('Image_MS_SJSM_2d_M',NumImages);
            ret            = strcat(ret,File_name1);
        else
            % look into this
            temp           = ret;   
            File_name1     = strcat('Image_MS_SJSM_Common_2d_M',NumImages);
            File_name2     = strcat('Image_MS_SJSM_Inn_2d_M', NumImages);
            ret            = strcat(temp,File_name1,'.jpg');
            ret1           = strcat(temp,File_name2,'.jpg');
        end
        
    case {'Image Data'}
        data_path      = strcat(root_path,'Face_Database\');
        data_path      = strcat(data_path, 'ImageClass');
        data_path      = strcat(data_path,ClassNum);
        data_path      = strcat(data_path, '\');
        fname          = dir(strcat(data_path,'*.jpg'));
        ret            = strcat(data_path,fname(index).name);

end