%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Pradeep Nagesh, Arizona State University, USA (2007-2008)  
%         pnagesh@asu.edu
%         Face Recognition using Joint Sparsity Models
%
% Description: Generate measurement matrix Phi and Basis matrix for 1D JSM.
% If any of (or both) the matrix(cies) are already saved, dont waste
% computations in generating the dense random/basis matrices, but simply
% load them. If not generate and save them so that next time they may be
% simply loaded. Only up to 5 images can be handles, otherwise size of Phi
% and basis matrix becomes overwhlemingly high for a nominal size of 32x32
% image. 
%
%  Input/Output 
%  Options: Input data structure containing various parameter settings.  
%  Refer to FR_Document.doc for details.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Phi shi_tilda] = Phi_Shi_TetaMat(Options)

N1 = Options.SizeImage(1);
N2 = Options.SizeImage(2);
SampleFraction = Options.SampleFraction;
NumImages = Options.M;

N = N1*N2;
K = round(SampleFraction/100*N);

idctmx = khronical_product(dctmtx(N1)');

Phi_Mat = GetPath_name('Phi',Options);

if (NumImages > 5)
    error('Not more than 5 images are supported for joint reconstruction');
end

for i=1:NumImages
    Phi_Num = strcat('Phi',num2str(i));
    try 
        load(Phi_Mat,Phi_Num);
    catch
        Phi1 = randn(K,N);
        Phi1 = orth(Phi1')';
        Phi2 = randn(K,N);
        Phi2 = orth(Phi2')';
        Phi3 = randn(K,N);
        Phi3 = orth(Phi3')';
        Phi4 = randn(K,N);
        Phi4 = orth(Phi4')';
        Phi5 = randn(K,N);
        Phi5 = orth(Phi5')';
        Phi6 = randn(K,N);
        Phi6 = orth(Phi6')';
        save(Phi_Mat,'Phi1', 'Phi2', 'Phi3', 'Phi4', 'Phi5', 'Phi6');
    end;        
end

if(NumImages == 2)
    Phi = [Phi1 zeros(K,N);...
          zeros(K,N) Phi2];
    clear Phi1;
    clear Phi2;
    shi_tilda = [idctmx idctmx zeros(N,N);...
                idctmx zeros(N,N) idctmx];
    
elseif(NumImages == 3)
    Phi = [Phi1 zeros(K,N) zeros(K,N);...
          zeros(K,N) Phi2 zeros(K,N);...
          zeros(K,N) zeros(K,N) Phi3];
    clear Phi1;
    clear Phi2;
    clear Phi3;
    shi_tilda = [idctmx idctmx zeros(N,N) zeros(N,N);...
                 idctmx zeros(N,N) idctmx zeros(N,N);...
                 idctmx zeros(N,N) zeros(N,N) idctmx;];    

elseif(NumImages == 4)
    Phi = [Phi1 zeros(K,N) zeros(K,N) zeros(K,N);...
           zeros(K,N) Phi2 zeros(K,N) zeros(K,N);... 
           zeros(K,N) zeros(K,N) Phi3 zeros(K,N);...
           zeros(K,N) zeros(K,N) zeros(K,N) Phi4;];
    clear Phi1;
    clear Phi2;
    clear Phi3;
    clear Phi4;
    shi_tilda = [idctmx idctmx zeros(N,N) zeros(N,N) zeros(N,N);...
                 idctmx zeros(N,N) idctmx zeros(N,N) zeros(N,N);...
                 idctmx zeros(N,N) zeros(N,N) idctmx zeros(N,N);...
                 idctmx zeros(N,N) zeros(N,N) zeros(N,N) idctmx];
    
elseif(NumImages == 5)
    Phi = [Phi1 zeros(K,N) zeros(K,N) zeros(K,N) zeros(K,N);...
           zeros(K,N) Phi2 zeros(K,N) zeros(K,N) zeros(K,N);... 
           zeros(K,N) zeros(K,N) Phi3 zeros(K,N) zeros(K,N);...
           zeros(K,N) zeros(K,N) zeros(K,N) Phi4 zeros(K,N);...
           zeros(K,N) zeros(K,N) zeros(K,N) zeros(K,N) Phi5];
    clear Phi1;
    clear Phi2;
    clear Phi3;
    clear Phi4;
    clear Phi5;
    shi_tilda = [idctmx idctmx zeros(N,N) zeros(N,N) zeros(N,N) zeros(N,N);...
                 idctmx zeros(N,N) idctmx zeros(N,N) zeros(N,N) zeros(N,N);...
                 idctmx zeros(N,N) zeros(N,N) idctmx zeros(N,N) zeros(N,N);...
                 idctmx zeros(N,N) zeros(N,N) zeros(N,N) idctmx zeros(N,N);...
                 idctmx zeros(N,N) zeros(N,N) zeros(N,N) zeros(N,N) idctmx];

end
    
    
    