%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Pradeep Nagesh, Arizona State University, USA (2007-2008)  
%         pnagesh@asu.edu
%         Face Recognition using Joint Sparsity Models
%
% Description: Generate measurement matrix Phi and Basis matrix for JSM. If
% any of (or both) the matrix(cies) are already saved, dont waste
% computations in generating the dense random/basis matrices, but simply
% load them. If not generate and save them so that next time they may be
% simply loaded.
%
%  Input/Output 
%  Options: Input data structure containing various parameter settings.  
%  Refer to FR_Document.doc for details.
%
%  operation: BJSM or SJSM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [JSM_P JSM_Basis] = Phi_Shi_TetaMat2D(Options,operation)

%if (Options.twoDReconstruction)
%    error('The 2D reconstruction option is not set');
%end

if nargin < 2
    operation = 'BJSM'
end
    
N = Options.SizeImage(1);
N2 = Options.SizeImage(2);
FormPhiIs = 0;
FormPhi_SJSM_Is = 0;
FormShitilda  = 0;
FormShi_SJSM = 0;
Phi = 0;

%if (N ~= N2)
%    error('In 2D, non-square inmages are not suppored');
%end

K = N;
temp = Options.M;
NumImages = temp(Options.ClassNum);
idctmx = dctmtx(N)';
I = eye(N);

Phi_Mat_Full = GetPath_name('Phi Full',Options);

if (strcmp(operation,'BJSM'))
    try
       load(Phi_Mat_Full,'Phi');
       if(size(Phi) ~= [K*NumImages N*NumImages])
           FormPhiIs = 1;
            for i=1:NumImages
                Phi_Mat = GetPath_name('Phi',Options);
                Phi_Mat = strcat(Phi_Mat,num2str(i));
                try 
                    load(Phi_Mat,Phi_Num);
                catch
                    P = randn(K,N);
                    P = orth(P')';
                    eval(strcat(sprintf('Phi%d',i),'= P;'));
                    save(Phi_Mat,sprintf('Phi%d',i));
                end        
            end
       end
    catch
        FormPhiIs =1;
        for i=1:NumImages
            Phi_Mat = GetPath_name('Phi',Options);
            Phi_Mat = strcat(Phi_Mat,num2str(i));
            try 
                load(Phi_Mat,Phi_Num);
            catch
                P = randn(K,N);
                P = orth(P')';
                eval(strcat(sprintf('Phi%d',i),'= P;'));
                save(Phi_Mat,sprintf('Phi%d',i));
            end        
        end
    end

    if FormPhiIs
        M = NumImages;
        Phi = zeros(K*M,N*M);

        for i=1:M
            for j=1:M
                if (i == j)
                    Phi((i-1)*K+1:i*K,(j-1)*N+1:j*N) = eval(sprintf('Phi%d',j));
                end
            end
        end
        save(Phi_Mat_Full,'Phi');
    end

    try
       load(Phi_Mat_Full,'shi_tilda');
       if(size(shi_tilda) ~= [K*NumImages N*(NumImages+1)])
           FormShitilda = 1;
       end
    catch
        shi_tilda = Form_shi_Matrix(idctmx,NumImages);
        try
            load(Phi_Mat_Full);
            save(Phi_Mat_Full,'shi_tilda', '-Append');
        catch
            save(Phi_Mat_Full,'shi_tilda');
        end        
    end

    if (FormShitilda)
        shi_tilda = Form_shi_Matrix(idctmx,NumImages);
        FormShitilda = 0;

        try
            load(Phi_Mat_Full);
            save(Phi_Mat_Full,'shi_tilda', '-Append');
        catch
            save(Phi_Mat_Full,'shi_tilda');
        end        
    end
    
    JSM_P = Phi;
    JSM_Basis = shi_tilda;
end

if (strcmp(operation,'SJSM'))
    try
       load(Phi_Mat_Full,'Phi_SJSM');
       if(size(Phi_SJSM) ~= [K*NumImages N*NumImages])
           FormPhi_SJSM_Is = 1;
       end
    catch
        FormPhi_SJSM_Is =1;
    end

    if FormPhi_SJSM_Is
        M = NumImages;
        Phi_SJSM = zeros(K*M,N*M);
        I = eye(K);

        for i=1:M
            for j=1:M
                if (i == j)
                    Phi_SJSM((i-1)*K+1:i*K,(j-1)*N+1:j*N) = I;
                end
            end
        end
        try
            load(Phi_Mat_Full);
            save(Phi_Mat_Full,'Phi_SJSM', '-Append');
        catch
            save(Phi_Mat_Full,'Phi_SJSM');
        end        
    end

    try
       load(Phi_Mat_Full,'shi_SJSM');
       if(size(shi_SJSM) ~= [K*NumImages N*NumImages])
           FormShi_SJSM = 1;
       end
    catch
        I = eye(K);
        shi_SJSM = Form_shi_Matrix(I,NumImages);
        try
            load(Phi_Mat_Full);
            save(Phi_Mat_Full,'shi_SJSM', '-Append');
        catch
            save(Phi_Mat_Full,'shi_SJSM');
        end        
    end

    if (FormShi_SJSM)
        I = eye(K);
        FormShitilda = 0;
        shi_SJSM = Form_shi_Matrix(I,NumImages);
        try
            load(Phi_Mat_Full);
            save(Phi_Mat_Full,'shi_SJSM', '-Append');
        catch
            save(Phi_Mat_Full,'shi_SJSM');
        end        
    end
    JSM_P = Phi_SJSM;
    JSM_Basis = shi_SJSM;
end

function shiMatrix = Form_shi_Matrix(Matrix,R)

[M L] = size(Matrix);

diagM = zeros(M*R,L*R);
shiMatrix = [];

for m=1:R
    for l=1:R
        if (m == l)
            diagM((m-1)*M+1:m*M,(l-1)*L+1:l*L) = Matrix;
        end
    end
end

shiMatrix = [ [repmat(Matrix,R,1)] diagM ]; 